/*
	User Info Database Check / Cleanup Utility for XFBB 7.01 or newer
	Copyright (c) 2001/2002 by Stewart Wilkinson (G0LGS) 
*/

#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <fcntl.h>
#include <stdarg.h>
#include <string.h> 
#include <time.h>
#include <ctype.h>
#include <getopt.h>
#include <errno.h>
#include <unistd.h>
#include <sys/signal.h>
#include <sys/stat.h>
#include <sys/wait.h>

#include "utils.h"

extern char *optarg;
extern int optind;
extern int opterr;
extern int getopt(int argc, char * const *argv, const char *optstring);

char data_path[128];
char conf_path[128];
char inf_sys[128];
char inf_new[128];
char inf_old[128];
char lang_sys[128];

int  lng = 0;
int  all = 0;
int  verbose = 0;
int  tidy = 0;
int  check = 0;
int  clean = 0;
int  list = 0;
int  rem_exc = 0;

int  nb;
int  nb_lang;
int  nb_buff;
int  deflang;

time_t time_now;
time_t time_expire;

#define _read read
#define _write write
#define O_BINARY 0
#define __a2__ __attribute__ ((packed, aligned(2)))

#define F_EXC	0x0001
#define F_LOC	0x0002
#define F_EXP	0x0004
#define F_SYS	0x0008
#define F_BBS	0x0010
#define F_PAG	0x0020
#define F_GST	0x0040
#define F_MOD	0x0080
#define F_PRV	0x0100
#define F_UNP	0x0200
#define F_NEW	0x0400
#define F_PMS	0x0800

#define EXC(flag)	(flag & F_EXC)
#define LOC(flag)	(flag & F_LOC)
#define EXP(flag)	(flag & F_EXP)
#define SYS(flag)	(flag & F_SYS)
#define BBS(flag)	(flag & F_BBS)
#define PAG(flag)	(flag & F_PAG)
#define GST(flag)	(flag & F_GST)
#define MOD(flag)	(flag & F_MOD)
#define PRV(flag)	(flag & F_PRV)
#define UNP(flag)	(flag & F_UNP)
#define NEW(flag)	(flag & F_NEW)
#define PMS(flag)	(flag & F_PMS)

typedef struct typindic
{
	char call[7];
	char num;
}
indicat;

typedef struct
{
	indicat indic;			/* 8  Callsign */
	indicat relai[8];		/* 64 Digis path */
	long	lastmes __a2__;		/* 4  Last L number */
	long	nbcon   __a2__;		/* 4  Number of connexions */
	long	hcon    __a2__;		/* 4  Last connexion date */
	long	lastyap __a2__;		/* 4  Last YN date */
	unsigned short	flags;		/* 2  Flags */
	unsigned short	on_base;	/* 2  ON Base number */
	unsigned char	nbl;		/* 1  Lines paging */
	unsigned char	lang;		/* 1  Language */
	long	newbanner __a2__;	/* 4  Last Banner date */
	unsigned short 	download  ;	/* 2  download size (KB) = 100 */
	char	free[20];		/* 20 Reserved */
	char	theme;			/* 1  Current topic */
	char	nom[18];		/* 18 Surname */
	char	prenom[13];		/* 13 Christian name */
	char	adres[61];		/* 61 Address */
	char	ville[31];		/* 31 City */
	char	teld[13];		/* 13 home phone */
	char	telp[13];		/* 13 modem phone */
	char	home[41];		/* 41 home BBS */
	char	qra[7];			/* 7  Qth Locator */
	char	priv[13];		/* 13 PRIV directory */
	char	filtre[7];		/* 7  LC choice filter */
	char	pass[13];		/* 13 Password */
	char	zip[9];			/* 9  Zipcode */
					/* Total : 360 bytes */
}
info;

typedef struct tulist
{
	char call[7];
	long loc;
	struct tulist *next;
}
ulist;

ulist *userlist;

#define FBBCONF "/etc/ax25/fbb.conf"

int ReadInit()
{
	int len, i;
	int end = 0;
	char buffer[256];
	char *scan;
	char *ptr;
	char key[20];
	char pathname[256];
	FILE *init_ptr;

	if( (ptr = getenv("FBBCONF")) == NULL ){
		strcpy( pathname, FBBCONF);
	}else{
		strcpy( pathname, ptr);
	}

	if( access( pathname, R_OK ) != 0 ){
		fprintf( stderr, "Cannot find %s\n", pathname );
		return 1;
	}

	if ((init_ptr = fopen (pathname, "r")) != NULL) {

		if( verbose ){
			fprintf( stdout, "Reading %s.\n", pathname );
		}

		while( !end && fgets(buffer, sizeof(buffer), init_ptr) ){

			scan = buffer;

			/* Eat Spaces */
			while (isspace(*scan))
				++scan;

			/* Comments & Blanks ignored */
			if (*scan == '#' || *scan == '\0' )
				continue;

			/* remove leading \n */
			len = strlen(scan);
			if( len && (scan[len-1] == '\n'))
				scan[len-1] = '\0';

			/* keyword */
			i=0;
			ptr = key;
			while( isgraph(*scan)) {
				if( i++ < 4 ) {
					*ptr++ = *scan;
				}
				++scan;
				if(*scan == '=')
					break;         
			}
			*ptr = '\0';

			/* find the '=' */
			scan = strchr(scan, '=');

			/* copy value */
			if(scan) {
				++scan;
				while( isspace(*scan))
					++scan;

				/* if 1st char is '"' then string */
				if( *scan == '"' ){
					++scan;
					ptr = strrchr(scan,'"');
					if(ptr)
						*ptr = '\0';
				}

				if( strcmp( key, "data" ) == 0 ) {
					strcpy( data_path, scan );

				}

				if( strcmp( key, "conf" ) == 0 ) {
					strcpy( conf_path, scan );

				}

			}
		}

		fclose(init_ptr);
  
	}else{
		fprintf( stderr, "Unable to open '%s'\n", pathname );
		return 1;
	}

	return 0;
}

void do_head(char *cmd)
{
	fprintf( stdout, "\nXFBB %s V" Version " " Vdate " (c) by Stewart Wilkinson (G0LGS)\n", strupr( cmd ) );
}

void do_usage(char *cmd)
{
	do_head( cmd );

	if( strcasecmp( cmd, "inf-tool" ) == 0 ) {

		fprintf( stderr, "\n\tUsage: %s [options]\n\n", strupr( cmd) );
		fprintf( stderr, "\t --list\t\t\tList all users\n" );
		fprintf( stderr, "\t --check\t\tShow what would be done\n" );
		fprintf( stderr, "\t --tidy\t\t\tRemove corrupt entries, reset counters etc.\n" );
		fprintf( stderr, "\t --clean=no_of_days\tRemove users that have not logged in for x days.\n" );
		fprintf( stderr, "\t --verbose\t\tShow more information.\n" );
		fprintf( stderr, "\t --remove-excluded\tRemove corrupt excluded entries.\n" );
		fprintf( stderr, "\t -s | --short\t\tShort list format (default).\n" );
		fprintf( stderr, "\t -l | --long\t\tLong list format.\n" );
		fprintf( stderr, "\t -v | --version\n" );

	}else{

		fprintf( stderr, "\n\tUsage: %s [options]\n\n", strupr(cmd) );
		fprintf( stderr, "\t -s | --short\t\tShort list format (default).\n" );
		fprintf( stderr, "\t -l | --long\t\tLong list format.\n" );
	}		

	fprintf( stderr, "\n" );
	
}


int Is_fbb(void) {

	int pid;

	pid = WEXITSTATUS(system("pidof xfbb >/dev/null 2>&1") );

	if( (pid != -1) && (pid != 0) ){
		pid = WEXITSTATUS( system("pidof xfbbd >/dev/null 2>&1") );

	}

	return(pid);
}


char *strflags (info * frec)
{
#define NB_FLAG 11
	int i;
	static char flag[NB_FLAG + 1];
 
	for (i = 0; i < NB_FLAG; i++)
		flag[i] = '.';
	flag[NB_FLAG] = '\0';
 
	if (NEW (frec->flags))
		flag[10] = 'I';
	if (UNP (frec->flags))
		flag[9] = 'U';
	if (MOD (frec->flags))
		flag[8] = 'M';
	if (EXC (frec->flags))
		flag[7] = 'E';
	if (PMS (frec->flags))
		flag[6] = 'F';
	if (LOC (frec->flags))
		flag[5] = 'L';
	if (EXP (frec->flags))
		flag[4] = 'X';
	if (SYS (frec->flags))
		flag[3] = 'S';
	if (BBS (frec->flags))
		flag[2] = 'B';
	if (PAG (frec->flags))
		flag[1] = 'P';
	if (PRV (frec->flags))
		flag[0] = 'R';
 
	return (flag);
}

int do_usrlist(char *file)
{
	info userrec;
	long count = 0L;
	long good = 0L;
	FILE *ptr;

	if( access( file, R_OK ) != 0 ){
		fprintf( stderr, "unable to find %s\n", file );
		return 1;
	}

	/* Open */
 	ptr = fopen( file, "rb");
	if ( ptr == NULL ) {
		fprintf( stderr, "Unable to open '%s'\n", file );
		return 1; 
	}

	fprintf( stdout, "\nCallsign  LastConnect #Cnct RPBSXLFEMUI Lg Name      Password     PRIV-Dir.\n" );

	while( fread(&userrec, sizeof(info), 1, ptr) && ! feof( ptr ) ){

		count++;
		if( *userrec.indic.call != '\0' ){

			good++;

			if( ! all ) {
				fprintf( stdout, "%-6s-%-2d ", userrec.indic.call, userrec.indic.num );
				fprintf( stdout, "%s ", strdt(userrec.hcon) );
				fprintf( stdout, "%5ld ", userrec.nbcon );
				fprintf( stdout, "%s ", strflags(&userrec) );
				fprintf( stdout, "%-12s ", strlen(userrec.prenom) ? userrec.prenom : "?" );
				fprintf( stdout, "%-12s ", userrec.pass );
				fprintf( stdout, "%-11s\n", userrec.priv );
			}

			if( lng ) {
				fprintf( stdout, "HomeBBS : %-40s\n", strlen(userrec.home) ? userrec.home : "?" );
				fprintf( stdout, "Zip     : %s\n\n", strlen(userrec.zip) ? userrec.zip : "?" );
			}

			if( all ) {
				fprintf( stdout, "Callsign: %-6s-%-2d\n", userrec.indic.call, userrec.indic.num );

				fprintf( stdout, "LastList: %ld\n", userrec.lastmes );
				fprintf( stdout, "Num Cons: %ld\n", userrec.nbcon );
				fprintf( stdout, "Last Con: %s\n", strdt_gb(userrec.hcon) );
				fprintf( stdout, "Last Yap: %s\n", strdt_gb(userrec.lastyap) );
				fprintf( stdout, "Flags   : %s\n", strflags(&userrec) );
				fprintf( stdout, "ON Base : %d\n", userrec.on_base );
				fprintf( stdout, "Lines   : %d\n", userrec.nbl );
				fprintf( stdout, "Language: %d\n", userrec.lang );
				fprintf( stdout, "Banner  : %s\n", strdt_gb(userrec.newbanner) );
				fprintf( stdout, "Download: %d KB\n", userrec.download );

				//fprintf( stdout, "Free    : %-19s\n", userrec.free );
				//fprintf( stdout, "Theme   : %2.2X\n", userrec.theme );

				fprintf( stdout, "1st Name: %-12s\n", userrec.prenom );
				fprintf( stdout, "Surname : %-12s\n", userrec.nom );

				fprintf( stdout, "Address : %-60s\n", userrec.adres );
				fprintf( stdout, "City    : %-30s\n", userrec.ville );
				fprintf( stdout, "Phone 1 : %-12s\n", userrec.teld );
				fprintf( stdout, "Phone 2 : %-12s\n", userrec.telp );

				fprintf( stdout, "HomeBBS : %-40s\n", userrec.home );
				fprintf( stdout, "QRA     : %-7s\n", userrec.qra );

				fprintf( stdout, "Priv-Dir: %-11s\n", userrec.priv );
				fprintf( stdout, "LC Flit : %-6s\n", strlen(userrec.filtre) ? userrec.filtre : "*" );
				fprintf( stdout, "Password: %-12s\n", userrec.pass );

				fprintf( stdout, "Zip     : %-8s\n", userrec.zip );

				fprintf( stdout, "\n" );

			}

		}
	}

	fclose( ptr );

	fprintf( stdout, "\n%ld Records\n", count );
	fprintf( stdout, "%ld Known Users\n", good );

	return 0;

}

long	dupl = 0L;
long	count = 0L;
long	old = 0L;
long	bad = 0L;
long	empty = 0L;

int Is_Bad( info * rec )
{
	int ret = 0;

	// Null callsign ?
	if( *rec->indic.call == '\0' ){
		empty++;
		if( verbose ){
			fprintf( stdout, "\tNull Record %ld\n", count );
		}
		ret = 1;

	// Check Callsign Structure
	}else if( Check_Call( rec->indic.call ) != 0 ){
		bad++;
		if( verbose ){
			fprintf( stdout, "\tInvalid Callsign (%s)\n", rec->indic.call );
		}
		ret = 1;

	// Invalid SSID ?
	}else if( ( rec->indic.num < 0 ) || ( rec->indic.num > 15 ) ){
		bad++;
		if( verbose ){
			fprintf( stdout, "\tInvalid SSID for %s\n", rec->indic.call );
		}
		ret = 1;

	// User EXCLUDED ?? - Skip rest of the Checks
	}else if( EXC(rec->flags) && ! rem_exc ) {
		ret = 0;

	// Check Hroute Structure
	}else if( Check_Home( rec->home ) != 0 ){
		bad++;
		if( verbose ){
			fprintf( stdout, "\tInvalid Hroute for %s\n", rec->indic.call );
		}
		ret = 1;

	}


	// TODO - Other possible checks.


	if( EXC(rec->flags) && rem_exc && (ret == 1) ) {

		if( check ) {
			fprintf( stdout, "WARNING: Corrupted entry for Excluded Callsign (%s) would be removed\n", rec->indic.call );
		}else{
			fprintf( stdout, "WARNING: Corrupted entry for Excluded Callsign (%s) removed\n", rec->indic.call );
		}
	}	

	return ret;
}


int Is_Old( info * rec )
{
	int ret = 0;

	// User EXCLUDED ??
	if( EXC(rec->flags) ) {
		ret = 0;

	}else if( rec->hcon <= time_expire ) {
		old++;
		ret = 1;
		if( verbose ){
			fprintf( stdout, "\tOld Entry for %s\n", rec->indic.call );
		}
	}

	return ret;
}

void insert( char *call, long pos )
{
	int cmp;
	ulist *ptr, *ptmp, *prec;

	prec = ptr = userlist;

	while( (ptr = ptr->next) != NULL ) {

		cmp = strcmp( call, ptr->call );
		if( cmp == 0 ) {
			fprintf( stderr, "Duplicate callsign (%s)\n", call );
			dupl++;
			return;

		}else if ( cmp < 0 )
			break;

		prec = ptr;
	}

	if( (ptmp = (ulist *) malloc( sizeof(ulist) ) ) == NULL ) {
		fprintf( stderr, "Memory Allocation Error\n" );
		exit(1);
	}

	ptmp->next = ptr;
	prec->next = ptmp;
	ptmp->loc = pos;
	strcpy( ptmp->call, call);
}

void fix_rec( info * rec )
{
	int i;
 
	if( ! check ) {
		rec->indic.call[6] = '\0';
		for (i = 0; i < 8; i++)
			rec->relai[i].call[6] = '\0';
		rec->nom[17] = '\0';
		rec->prenom[12] = '\0';
		rec->adres[60] = '\0';
		rec->ville[30] = '\0';
		rec->teld[12] = '\0';
		rec->telp[12] = '\0';
		rec->home[40] = '\0';
		rec->qra[6] = '\0';
		rec->priv[12] = '\0';
		rec->filtre[6] = '\0';
		rec->pass[12] = '\0';
		rec->zip[8] = '\0';
	}

	// Reset Nb Connections ?
	if( ( rec->nbcon < 0 ) || ( rec->nbcon >= 100000L ) ){

		if( verbose ){
			fprintf( stdout, "\tNo Connections : %ld out of range for %s\n", rec->nbcon, rec->indic.call );
		}

		if( ! check )
			rec->nbcon = 0;

	}

	// Reset Nb Lines (OL) ?
	if( ( rec->nbl < (unsigned char) 4 ) || ( rec->nbl > (unsigned char) 50 ) ){

		if( verbose ){
			fprintf( stdout, "\tNo Lines (OL) : %d out of range for %s\n", rec->nbl, rec->indic.call );
		}

		if( ! check )
			rec->nbl = (unsigned char) 25;
	}

	// Reset Lang ?
	if( ( rec->lang > (unsigned char) (nb_lang-1) ) ){

		if( verbose ){
			fprintf( stdout, "\tInvalid Language (%d) for %s\n", rec->lang, rec->indic.call );
		}

		if( ! check )
			rec->lang = (unsigned char) deflang;
	}

}


int do_tidy(void)
{
	info	userrec;
	FILE	*infsys, *infnew;

	ulist *ptr;

	if( ! verbose ) {
		fprintf( stdout, "\nChecking .." );
		fflush( stdout );
	}

	if( ! copy_ ( inf_sys, inf_old ) ){
		fprintf( stderr, "ERROR: Unable to copy '%s' to '%s'\n", inf_sys, inf_old );
		/* Enable CTRL-C */
		signal( SIGINT, SIG_DFL );
		return 1; 
	}

	/* Open */
 	infsys = fopen( inf_sys, "rb");
	if ( infsys == NULL ) {
		fprintf( stderr, "Unable to open '%s'\n", inf_sys );
		/* Enable CTRL-C */
		signal( SIGINT, SIG_DFL );
		return 1; 
	}

	/* Open */
 	infnew = fopen( inf_new, "wb");
	if ( infnew == NULL ) {
		fprintf( stderr, "Unable to create '%s'\n", inf_new );
		/* Enable CTRL-C */
		signal( SIGINT, SIG_DFL );
		return 1; 
	}

	/* Initialize list */
	if( ( userlist = (ulist *) malloc( sizeof( ulist ) ) ) == NULL  ) {
		fprintf( stderr, "Memory Allocation Error\n" );
		return 1;
	}
	userlist->next = NULL;

	count=0;
	rewind( infsys );
	while( fread(&userrec, sizeof(info), 1, infsys) && ! feof( infsys ) ){
		
		if( tidy && Is_Bad( &userrec ) ) {
			// Remove entry

		}else if ( clean && Is_Old( &userrec ) ) {
			// Remove entry

		}else{
			// Keep Record
			insert( userrec.indic.call, count );	
		}

		count++;
		if( ! verbose && ! (count % 10) ){
			fprintf( stdout, ".");
			fflush( stdout );
		}

	}
	
	ptr = userlist;
	while( (ptr = ptr->next) != NULL) {
	
		fseek( infsys, ptr->loc * (long) sizeof( info ), 0 );
		fread(&userrec, sizeof(info), 1, infsys);

		if( tidy ) {
			fix_rec( &userrec );
		}

		if( ( fwrite( &userrec, sizeof(info), 1, infnew ) == 0 ) ) {

			fprintf( stderr, "Error: writing to %s\n", inf_new );
			fclose( infsys );
			fclose( infnew );

			/* Enable CTRL-C */
			signal( SIGINT, SIG_DFL );

			return 1;
		}

	}

	fprintf( stdout, "\n" );	

	fclose( infsys );
	fclose( infnew );

	if( check ) {

		fprintf( stdout, "%ld Entries Checked\n", count );	

		if( tidy ) {
			fprintf( stdout, "%ld Empty entries found\n", empty );
			fprintf( stdout, "%ld Invalid entries found\n", bad );
			fprintf( stdout, "%ld Duplicate entries found\n", dupl );
		}

		if( clean ) {
			fprintf( stdout, "%ld Old entries found\n", old );
		}

	} else {

		/* replace original inf.sys with copy of inf.new */
		if( ! copy_ ( inf_new, inf_sys ) ){
			fprintf( stderr, "ERROR: Unable to copy '%s' to '%s'\n", inf_new, inf_sys );
			fprintf( stderr, "Please copy the file manually before re-starting XFBB\n" );
		}

		fprintf( stdout, "%ld Entries Checked\n", count );	

		if( tidy ) {
			fprintf( stdout, "%ld Empty entries deleted\n", empty );
			fprintf( stdout, "%ld Invalid entries deleted\n", bad );
			fprintf( stdout, "%ld Duplicate entries deleted\n", dupl );
		}

		if( clean ) {
			fprintf( stdout, "%ld Old entries deleted\n", old );
		}

	}

	/* Enable CTRL-C */
	signal( SIGINT, SIG_DFL );
	sprintf( inf_sys, "%s/inf.sys", data_path );

	fprintf( stdout, "Finished.\n" );	
	return 0;
}




int main(int argc, char *argv[])
{
	char Cmd[128];
	char *ap;
	int ch;
	int option_index = 0;
	static struct option long_options[] =
	{
	{ "help", 0, 0, 'h' },
	{ "verbose", 0, 0, 0 },
	{ "clean", 1, 0, 0 },
	{ "check", 0, 0, 0 },
	{ "list", 0, 0, 0 },
	{ "all", 0, 0, 0 },
	{ "long", 0, 0, 'l' },
	{ "short", 0, 0, 's' },
	{ "remove-excluded", 0, 0, 0 },
	{ "tidy", 0, 0, 0 },
	{ "version", 0, 0, 'v' },
	{ 0, 0, 0, 0 }
	};

	char *arg;
	int err;
	int fbbpid;

	FILE *langsys;
	char buffer[80];
	int end = 0;

	strcpy( Cmd, argv[0] );
	if( (ap = rindex( Cmd, '/') ) != NULL ) {
		ap++;
	}else{
		ap=Cmd;
	}

	while( (ch = getopt_long( argc, argv, "?hHvVlLsS", long_options, &option_index ) ) != (-1) ) {

		switch(ch){

			case 0:
				if( strcmp(long_options[option_index].name, "verbose" ) == 0 ){
					verbose=1;

				}else if( strcmp(long_options[option_index].name, "remove-excluded" ) == 0 ){
					rem_exc=1;

				}else if( strcmp(long_options[option_index].name, "tidy" ) == 0 ){
					tidy=1;

				}else if( strcmp(long_options[option_index].name, "check" ) == 0 ){
					check=1;

				}else if( strcmp(long_options[option_index].name, "list" ) == 0 ){
					list=1;

				}else if( strcmp(long_options[option_index].name, "all" ) == 0 ){
					all=1;
					lng=0;

				}else if( strcmp(long_options[option_index].name, "clean" ) == 0 ){
					if( optarg ) {
						arg = optarg;
						while(*arg){
							if(!isdigit(*arg)){
								do_head( ap );
								fprintf( stderr, "clean value muse be numeric\n" );
								return 1;
							}
							arg++;
						}
						sscanf( optarg, "%d", &clean );
					}
				}

				break;

			case '?':
			case 'h':
			case 'H':
				do_usage( ap );
				return 1;

			case 'v':
			case 'V':
				do_head( ap );
				return 1;

			case 'l':
			case 'L':
				lng=1;
				all=0;
				break;

			case 's':
			case 'S':
				lng=0;
				all=0;
				break;

			default:
				return 1;
		}
	}

	do_head( ap );

	if( ReadInit() != 0 ){
		fprintf( stderr, "Error Processing Config file.\n" );
		exit(1);
        }

	sprintf( inf_sys, "%s/inf.sys", data_path );
	sprintf( inf_new, "%s/inf.new", data_path );
	sprintf( inf_old, "%s/inf.old", data_path );

	sprintf( lang_sys, "%s/langue.sys", conf_path );

	if( access( inf_sys, W_OK ) != 0 ){
		fprintf( stderr, "Cannot find '%s'\n", inf_sys );
		exit(1); 
	}

	if( access( lang_sys, R_OK ) != 0 ){
		fprintf( stderr, "Cannot find '%s'\n", lang_sys );
		exit(1); 
	}
	
	// Check langue.sys
 	langsys = fopen( lang_sys, "rb");
	if ( langsys == NULL ) {
		fprintf( stderr, "Unable to open '%s'\n", lang_sys );
		return 1; 
	}

	end = 0;
	while( !end && fgets( buffer, sizeof(buffer), langsys ) ){

		if( *buffer == '#' )
			continue;

		if( isdigit(*buffer) ) {

			nb = sscanf( buffer, "%d %d %d", &nb_lang, &nb_buff, &deflang );

			if( nb != 3 ){
				fprintf( stderr, "Error in %s\n", lang_sys );
				return 1; 
			}

			if( deflang > nb_lang )
				deflang = 0;
			else
				deflang--;

			end++;

		}else{
			fprintf( stderr, "Error in %s\n", lang_sys );
			return 1; 
		}

	}

	fclose( langsys );


	if( (strcasecmp( ap, "inf-tool" ) == 0 ) ) {

		if( ! list && ! tidy && ! clean ){
			fprintf( stdout, "Nothing to do.\n" );
			fprintf( stdout, "\t at least one of --list --tidy or --clean must be specified.\n" );
			fprintf( stdout, "\n see --help\n" );
			return 1;
		}

		if( list ) {
			do_usrlist(inf_sys);
			return 0;
		}

		if( verbose ){
			fprintf( stdout, "Checking if XFBB is active" );
			fflush( stdout );
		}

		fbbpid = Is_fbb();

		if( fbbpid == 127 ) {
			if( verbose ){
				fprintf( stdout, " - Unable to determine\n");
			}
			fprintf(stderr, "\007Error: Unable to determine if XFBB is currently active.\n\n" );
			return 2;
		}

		if( fbbpid == 0 ) {
			if( verbose ){
				fprintf( stdout, " - YES\n");
			}
			fprintf(stderr, "\007Error: XFBB is currently active - Cannot modify user database at this time.\n\n" );
			return 2;
		}

		if( verbose ){
			fprintf( stdout, " - No\n");
		}

		time_now = time(NULL);
		time_expire = time_now - (time_now % 86400) - (clean * 86400);	

		err = do_tidy();

	}else if( strcasecmp(ap, "listusr") == 0 ) {

		do_usrlist(inf_sys);
		err = 0;

	}else{
		fprintf( stderr, "name %s is unknown\n\007", ap );
		err = 255;
	}

	exit(err);
}

